require 'rubygems'
require 'nokogiri'
require 'open-uri'
require 'GoogleFinance/HistoricalPrices'

module GoogleFinance

  #
  # Google Company class
  class Company

    attr_accessor :ticker, :name, :employees, :prices

    #
    # get the ticker with the initialize
    def initialize(ticker)
      @ticker = ticker
    end

    #
    # Get company name as in Google Finance
    def name
      name_url = "https://google.com/finance?q=" + @ticker
      page = Nokogiri::HTML(open(name_url))

      # use xpath to get the string, remove the end newline
      @name = page.xpath("//meta[@itemprop='name']/@content").to_s.gsub("\n", "")
      @name
    end

    #
    # Parse the number of employees
    def employees
      return @employees unless @employees.nil?

      name_url   = "https://google.com/finance?q=" + @ticker
      page       = Nokogiri::HTML(open(name_url))
      @employees = page.xpath("//div[@class='sfe-section']/table/tr[6]/td[@class='period']").inner_html.gsub(/\n|,|-/, "").to_i
      @employees

    end # employees

    #
    # load the historical prices iwth the current ticker
    def get_historical_prices(date_start, date_end)

      @prices = GoogleFinance::HistoricalPrices.new(@ticker, date_start, date_end)

    end # get_historical_prices

    #
    # get the outstanding share count
    def shares
      return @shares unless @shares.nil?

      name_url = "https://google.com/finance?q=" + @ticker
      page     = Nokogiri::HTML(open(name_url))
      share    = page.xpath("//table[@class='snap-data']/tr[3]/td")[3].inner_html.gsub(/\n|,|-/, "")

      # if the thing says 'M' then it's million, if it's 'B' then billions, obviously
      if    share.match(/M$/) then
        @shares = share.gsub("M", "").to_f
      elsif share.match(/B$/) then
        @shares = share.gsub("B", "").to_f * 1000
      end
      @shares
    end


  end # class Company

end #module
